with Ada.Text_IO, Logicals, Globals, Ints, Rationals, Flts;
use Ada.Text_IO, Logicals, Globals, Ints, Rationals, Flts;

pragma Elaborate_All (Ada.Text_IO,Logicals,Globals,Ints,Rationals,Flts);

generic
  
-------------------- begin_include scalars Real

  type Real is private;

--- basic
  with function Info(Dummy: Real) return Scalar_Info is <>;
  with function IsSharp(S: Real) return Boolean is <>;
  with function IsNumbers(S: Real) return Boolean is <>;
  with function IsZero(S: Real) return Boolean is <>;
  with function "="(S1,S2: Real) return Boolean is <>;
  with procedure SetZero(S: in out Real) is <>;
  with procedure Copy(S1: in Real; S2: in out Real) is <>;
  with procedure Swap(S1,S2: in out Real) is <>;

--- sets
  with function Center0(S: Real) return Boolean is <>;
  with function Contains0(S: Real) return Logical is <>;
  with function Contains(S1,S2: Real) return Logical is <>; -- S1 constains S2
  with procedure BallAt0(R: in Flt; S: in out Real) is <>;
  with function BallAt0(R: Flt) return Real is <>;
  with procedure DiskAt0(R: in Flt; S: in out Real) is <>;
  with function DiskAt0(R: Flt) return Real is <>;
  with procedure ToErr(S: in out Real) is <>;
  with procedure ToErr(S1: in Real; S2: in out Real) is <>;
  with function ToErr(S: Real) return Real is <>;
  with procedure Center(S: in out Real) is <>;
  with procedure Center(S1: in Real; S2: in out Real) is <>;
  with function Center(S: Real) return Real is <>;
  with procedure CenterDisk(S: in out Real) is <>;
  with procedure CenterDisk(S1: in Real; S2: in out Real) is <>;
  with function CenterDisk(S: Real) return Real is <>;
  with procedure ModCenter(S: in out Real) is <>;
  with procedure ModCenter(S1: in Real; S2: in out Real) is <>;
  with function ModCenter(S: Real) return Real is <>;
  with procedure ModNumbers(S: in out Real) is <>;
  with procedure ModNumbers(S1: in Real; S2: in out Real) is <>;
  with function ModNumbers(S: Real) return Real is <>;
  with procedure ErrMult(R: in Radius; S: in out Real) is <>;
  with procedure Union(S1: in Real; S2: in out Real) is <>;
  with function Union(S1,S2: Real) return Real is <>;
  with procedure Intersection(S1: in Real; S2: in out Real; Empty: out Logical) is <>;

--- order
  with function Sign(S: Real) return Integer is <>;
  with function Compare(S1,S2: Real) return Integer is <>;
  with function "<"(S1,S2: Real) return Boolean is <>;
  with function "<="(S1,S2: Real) return Boolean is <>;
  with function ">"(S1,S2: Real) return Boolean is <>;
  with function ">="(S1,S2: Real) return Boolean is <>;
  with procedure Min(S1: in Real; S2: in out Real) is <>;
  with procedure Min(S1,S2: in Real; S3: in out Real) is <>;
  with function Min(S1,S2: Real) return Real is <>;
  with procedure Max(S1: in Real; S2: in out Real) is <>;
  with procedure Max(S1,S2: in Real; S3: in out Real) is <>;
  with function Max(S1,S2: Real) return Real is <>;
  with function Inf(S: Real) return Flt is <>; -- inf of real part
  with function Sup(S: Real) return Flt is <>; -- sup of real part

--- arithmetic
  with procedure Neg(S: in out Real) is <>;
  with procedure Neg(S1: in Real; S2: in out Real) is <>;
  with function "-"(S: Real) return Real is <>;
  with procedure Add(I: in Integer; S: in out Real) is <>;
  with procedure Add(S1: in Real; S2: in out Real) is <>;
  with procedure Sum(S1,S2: in Real; S3: in out Real) is <>;
  with function "+"(S1,S2: Real) return Real is <>;
  with procedure Sub(S1: in Real; S2: in out Real) is <>;
  with procedure Diff(S1,S2: in Real; S3: in out Real) is <>;
  with function "-"(S1,S2: Real) return Real is <>;
  with procedure Mult(R: in Flt; S,Tmp: in out Real) is <>;
  with procedure Mult(R: in Flt; S: in out Real) is <>;
  with procedure Prod(R: in Flt; S1: in Real; S2: in out Real) is <>;
  with function "*"(R: Flt; S: Real) return Real is <>;
  with procedure AddProd(R: in Flt; S1: in Real; S2,Tmp: in out Real) is <>;
  with procedure AddProd(R: in Flt; S1: in Real; S2: in out Real) is <>;
  with procedure Mult(Q: in Rational; S: in out Real) is <>;
  with procedure Mult(S1: in Real; S2: in out Real) is <>;
  with procedure Prod(S1,S2: in Real; S3: in out Real) is <>;
  with function "*"(S1,S2: Real) return Real is <>;
  with procedure AddProd(S1,S2: in Real; S3,Tmp: in out Real) is <>;
  with procedure AddProd(S1,S2: in Real; S3: in out Real) is <>;
  with procedure SubProd(S1,S2: in Real; S3,Tmp: in out Real) is <>;
  with procedure Div(R: in Flt; S,Tmp: in out Real) is <>;
  with procedure Div(R: in Flt; S: in out Real) is <>;
  with procedure Quot(S1: in Real; R: in Flt; S2: in out Real) is <>;
  with function "/"(S: Real; R: Flt) return Real is <>;
  with procedure Div(S1: in Real; S2: in out Real) is <>;
  with procedure Quot(S1,S2: in Real; S3: in out Real) is <>;
  with function "/"(S1,S2: Real) return Real is <>;
  with procedure Inv(S1: in Real; S2: in out Real) is <>;
  with function Inv(S: Real) return Real is <>;
  with function "**"(S: Real; I: Integer) return Real is <>;

--- fun
  with function IsReal(S: Real) return Boolean is <>;
  with procedure Adjoint(S: in out Real) is <>;
  with procedure Adjoint(S1: in Real; S2: in out Real) is <>;
  with function Adjoint(S: Real) return Real is <>;
  with procedure Real_Part(S: in out Real) is <>;
  with procedure Imag_Part(S: in out Real) is <>;
  with procedure Eval0(S: in out Real) is <>;
  with function "abs"(S: Real) return Real is <>;
  with procedure Norm(S1: in Real; S2: in out Real) is <>;
  with function Norm(S: Real) return Real is <>;
  with function MaxNorm(S: Real) return Radius is <>;
  with procedure Sqr(S: in out Real) is <>;
  with function Sqr(S: Real) return Real is <>;
  with procedure Sqrt(S1: in Real; S2: in out Real) is <>;
  with function Sqrt(S: Real) return Real is <>;
  with procedure Root(K: in Positive; S1: in Real; S2: in out Real) is <>;
  with function Root(K: Positive; S: Real) return Real is <>;
  with procedure Exp(S1: in Real; S2: in out Real) is <>;
  with function Exp(S: Real) return Real is <>;
  with procedure ArcCos(S1: in Real; S2: in out Real) is <>;
  with function ArcCos(S: Real) return Real is <>;
  with procedure ArcSin(S1: in Real; S2: in out Real) is <>;
  with function ArcSin(S: Real) return Real is <>;
  with procedure Cos(S1: in Real; S2: in out Real) is <>;
  with function Cos(S: Real) return Real is <>;
  with procedure Sin(S1: in Real; S2: in out Real) is <>;
  with function Sin(S: Real) return Real is <>;
  with procedure Log(S1: in Real; S2: in out Real) is <>;
  with function Log(S: Real) return Real is <>;
  with procedure Simple_Random(S: in out Real) is <>;
  with procedure QPower(Q: in Rational; S1: in Real; S2: in out Real) is <>;
  with procedure Roots2(B,C: in Real; U1,U2,V: in out Real) is <>;
  with procedure Roots3(B,C,D: in Real; U0,U1,U2,V: in out Real) is <>;

--- conversion and i/o
  with function Approx(S: Real) return Flt is <>;
  with procedure Assign(K: in Integer; S: in out Real) is <>;
  with procedure Assign(Q: in Rational; S: in out Real) is <>;
  with procedure Assign(R: in Flt; S: in out Real) is <>;
  with function Scal(I: Integer) return Real is <>;
  with function Scal(Q: Rational) return Real is <>;
  with function Scal(R: Flt) return Real is <>;
  with procedure Enclose(R1,R2: in Flt; S: in out Real) is <>;
  with function Enclose(R1,R2: Flt) return Real is <>;
  with procedure Show1(N: in String; S: in Real; Whatever: in Boolean := True) is <>;
  with procedure Show2(N: in String; S1,S2: in Real; Whatever: in Boolean := True) is <>;
  with procedure Put(F: in File_Type; S: in Real; Decimal: in Boolean := False) is <>;
  with procedure Get(F: in File_Type; S: in out Real; Decimal: in Boolean := False) is <>;

--- misc
  with function Get_Precision(R: Real) return Positive is <>;
  with procedure Proper_Rounding(Dummy: in Real) is <>;
  with procedure Free_Cache(S: in Real) is <>;
-------------------- end_include scalars
  
package Complexs is

  pragma Elaborate_Body;
  
  type Complex is
    record
      Re: Real;
      Im: Real;
    end record;
  
  procedure NOP(Dummy: in out Complex);
  
--- basic
  function Info(Dummy: Complex) return Scalar_Info;
  function IsSharp(Z: Complex) return Boolean;
  function IsNumbers(Z: Complex) return Boolean;
  function IsZero(Z: Complex) return Boolean;
  function "="(Z1,Z2: Complex) return Boolean;
  procedure SetZero(Z: in out Complex);
  procedure Copy(Z1: in Complex; Z2: in out Complex);
  procedure Swap(Z1,Z2: in out Complex);

--- sets
  function Center0(Z: Complex) return Boolean;          --- symmetric about 0
  function Contains0(Z: Complex) return Logical;
  function Contains(Z1,Z2: Complex) return Logical;
  procedure BallAt0(R: in Flt; Z: in out Complex);
  function BallAt0(R: Flt) return Complex;              --- contains ball |z| <= |R|
  procedure DiskAt0(R: in Flt; Z: in out Complex);
  function DiskAt0(R: Flt) return Complex;              --- contains disk |z| <= |R|
  procedure ToErr(Z: in out Complex);                   --- 0-centered set containing Z
  procedure ToErr(Z1: in Complex; Z2: in out Complex);
  function ToErr(Z: Complex) return Complex;
  procedure Center(Z: in out Complex);
  procedure Center(Z1: in Complex; Z2: in out Complex);
  function Center(Z: Complex) return Complex;
  procedure CenterDisk(Z: in out Complex);
  procedure CenterDisk(Z1: in Complex; Z2: in out Complex);
  function CenterDisk(Z: Complex) return Complex;
  procedure ModCenter(Z: in out Complex);
  procedure ModCenter(Z1: in Complex; Z2: in out Complex);
  function ModCenter(Z: Complex) return Complex;
  procedure ModNumbers(Z: in out Complex);
  procedure ModNumbers(Z1: in Complex; Z2: in out Complex);
  function ModNumbers(Z: Complex) return Complex;
  procedure ErrMult(R: in Radius; Z: in out Complex);
  procedure Union(Z1: in Complex; Z2: in out Complex);
  function Union(Z1,Z2: Complex) return Complex;
  procedure Intersection(Z1: in Complex; Z2: in out Complex; Empty: out Logical);

--- order: undefined if non-real arguments
  function Sign(Z: Complex) return Integer;
  function Compare(Z1,Z2: Complex) return Integer;
  function "<"(Z1,Z2: Complex) return Boolean;
  function "<="(Z1,Z2: Complex) return Boolean;
  function ">="(Z1,Z2: Complex) return Boolean;
  function ">"(Z1,Z2: Complex) return Boolean;
  procedure Min(Z1: in Complex; Z2: in out Complex);
  procedure Min(Z1,Z2: in Complex; Z3: in out Complex);
  function Min(Z1,Z2: Complex) return Complex;
  procedure Max(Z1: in Complex; Z2: in out Complex);
  procedure Max(Z1,Z2: in Complex; Z3: in out Complex);
  function Max(Z1,Z2: Complex) return Complex;
  function Inf(Z: Complex) return Flt; -- inf of real part
  function Sup(Z: Complex) return Flt; -- sup of real part

--- addition and multiplication etc.
  procedure Neg(Z: in out Complex);
  procedure Neg(Z1: in Complex; Z2: in out Complex);
  function "-"(Z: Complex) return Complex;
  procedure Add(I: in Integer; Z: in out Complex);
  procedure Add(Z1: in Complex; Z2: in out Complex);
  procedure Sum(Z1,Z2: in Complex; Z3: in out Complex);
  function "+"(Z1,Z2: Complex) return Complex;
  procedure Sub(Z1: in Complex; Z2: in out Complex);
  procedure Diff(Z1,Z2: in Complex; Z3: in out Complex);
  function "-"(Z1,Z2: Complex) return Complex;
  procedure Mult(R: in Flt; Z,Tmp: in out Complex);
  procedure Mult(R: in Flt; Z: in out Complex);
  procedure Prod(R: in Flt; Z1: in Complex; Z2: in out Complex);
  function "*"(R: Flt; Z: Complex) return Complex;
  procedure AddProd(R: in Flt; Z1: in Complex; Z2,Tmp: in out Complex);
  procedure AddProd(R: in Flt; Z1: in Complex; Z2: in out Complex);
  procedure Mult(Q: in Rational; Z: in out Complex);
  procedure Mult(Z1: in Complex; Z2: in out Complex);
  procedure Prod(Z1,Z2: in Complex; Z3: in out Complex);
  function "*"(Z1,Z2: Complex) return Complex;
  procedure AddProd(Z1,Z2: in Complex; Z3,Tmp: in out Complex);
  procedure AddProd(Z1,Z2: in Complex; Z3: in out Complex);
  procedure SubProd(Z1,Z2: in Complex; Z3,Tmp: in out Complex);
  procedure Div(R: in Flt; Z,Tmp: in out Complex);
  procedure Div(R: in Flt; Z: in out Complex);
  procedure Quot(Z1: in Complex; R: in Flt; Z2: in out Complex);
  function "/"(Z: Complex; R: Flt) return Complex;
  procedure Div(Z1: in Complex; Z2: in out Complex);
  procedure Quot(Z1,Z2: in Complex; Z3: in out Complex);
  function "/"(Z1,Z2: Complex) return Complex;
  procedure Inv(Z1: in Complex; Z2: in out Complex);
  function Inv(Z: Complex) return Complex;
  function "**"(Z: Complex; I: Integer) return Complex;

--- functions
  function IsReal(Z: Complex) return Boolean;
  procedure Adjoint(Z: in out Complex);
  procedure Adjoint(Z1: in Complex; Z2: in out Complex);
  function Adjoint(Z: Complex) return Complex;
  procedure Real_Part(Z: in out Complex);
  procedure Imag_Part(Z: in out Complex);
  procedure Eval0(Z: in out Complex);
  procedure Norm(Z1: in Complex; Z2: in out Complex);
  function Norm(Z: Complex) return Complex;
  function "abs"(Z: Complex) return Complex renames Norm;
  function MaxNorm(Z: Complex) return Radius;
  procedure Sqr(Z: in out Complex);
  function Sqr(Z: Complex) return Complex;
  procedure Sqrt(Z1: in Complex; Z2: in out Complex);
  function Sqrt(Z: Complex) return Complex;
  procedure Root(K: in Positive; Z1: in Complex; Z2: in out Complex);
  function Root(K: Positive; Z: Complex) return Complex;
  procedure Exp(Z1: in Complex; Z2: in out Complex);
  function Exp(Z: Complex) return Complex;
  procedure ArcCos(Z1: in Complex; Z2: in out Complex);
  function ArcCos(Z: Complex) return Complex;
  procedure ArcSin(Z1: in Complex; Z2: in out Complex);
  function ArcSin(Z: Complex) return Complex;
  procedure Cos(Z1: in Complex; Z2: in out Complex);
  function Cos(Z: Complex) return Complex;
  procedure Sin(Z1: in Complex; Z2: in out Complex);
  function Sin(Z: Complex) return Complex;
  procedure Log(Z1: in Complex; Z2: in out Complex);
  function Log(Z: Complex) return Complex;
  procedure Simple_Random(Z: in out Complex);
  procedure QPower(Q: in Rational; Z1: in Complex; Z2: in out Complex);
  procedure Roots2(B,C: in Complex; U1,U2,V: in out Complex);
  procedure Roots3(B,C,D: in Complex; U0,U1,U2,V: in out Complex);

--- conversion and i/o
  function Approx(Z: Complex) return Flt;
  procedure Assign(I: in Integer; Z: in out Complex);
  procedure Assign(Q: in Rational; Z: in out Complex);
  procedure Assign(R: in Flt; Z: in out Complex);
  function Scal(I: Integer) return Complex;
  function Scal(Q: Rational) return Complex;
  function Scal(R: Flt) return Complex;
  procedure Enclose(R1,R2: in Flt; Z: in out Complex);
  function Enclose(R1,R2: Flt) return Complex;
  procedure Show1(N: in String; Z: in Complex; Hide0: in Boolean := True);
  procedure Show2(N: in String; Z1,Z2: in Complex; Hide0: in Boolean := True);
  procedure Put(F: in File_Type; Z: in Complex; Decimal: in Boolean := False);
  procedure Get(F: in File_Type; Z: in out Complex; Decimal: in Boolean := False);
  procedure Write(FileName: in String; Z: in Complex; Decimal: in Boolean := False);
  procedure Read(FileName: in String; Z: in out Complex; Decimal: in Boolean := False);

--- misc
  function Get_Precision(Z: Complex) return Positive;
  procedure Proper_Rounding(Z: in Complex);
  procedure Free_Cache(Z: in Complex);

--- additional functions
  procedure Norm(Z: in Complex; S: in out Real);
  function Norm(Z: Complex) return Real;
  procedure Mult(S: in Real; Z: in out Complex);
  function "*"(S: Real; Z: Complex) return Complex;
  procedure IMult(Z: in out Complex);    --- multiply by i
  procedure NegIMult(Z: in out Complex); --- multiply by -i
  
  procedure Polar(X,Y: in Real; Rad,Phi: in out Real);
  procedure Polar(Z: in Complex; Rad,Phi: in out Real);
  procedure UnPolar(Rad,Phi: in Real; Z: in out Complex);
  function UnPolar(Rad,Phi: Real) return Complex;
  
private

  RZero:    constant Real := Scal(Zero);
  RQuarter: constant Real := Scal(Quarter);
  RInfo: constant Scalar_Info := Info(RZero);
  RTrunc: constant Boolean := RInfo.IsNumeric;

  CInfo: constant Scalar_Info := (RepType    => RInfo.RepType,
                                  IsStandard => RInfo.IsStandard,
                                  IsNumeric  => RInfo.IsNumeric,
                                  IsComplex  => True,
                                  NReps      => 2,
                                  RMode      => RInfo.RMode);
  
  procedure Norm(X,Y: in Real; S,Tmp: in out Real);
  procedure Norm(Z: in Complex; S,Tmp: in out Real);
  procedure UPolar(X,Y: in Real; Rad,Phi,Tmp: in out Real);
  procedure RPolar(X,Y: in Real; Rad,Phi,Tmp: in out Real);
  procedure IPower(I: in Integer; S1: in Complex; S2: in out Complex);
  
end Complexs;
