with Strings, Flts.Plotting;
use Strings, Flts.Plotting;

pragma Elaborate_All (Strings,Flts.Plotting);

package body Fouriers1.Green is
   
   procedure SetRW(R: in Flt) is
   begin
      Copy(Scal(R),RW);
      CC1 := MakeCoeff(RW,RW);
   end SetRW;
   
   procedure SetRW(R1,R2,A: in Flt) is
      M: Matrix2;
   begin
      Copy(Scal(R1),Rw);
      Copy(Scal(R2),Rw2);
      Copy(Scal(A),Alpha);
      CC1 := MakeCoeff(RW,RW);
      CC2 := MakeCoeff(RW2,RW2);
      CC3 := MakeCoeff(RW,RW2);
      for K in 0..CC1'Last loop
	 M(1,1) := CC1(K);
	 M(1,2) := CC3(K);
	 M(2,1) := CC3(K);
	 M(2,2) := CC2(K);
	 Invert(M);
	 CC1i(K) := M(1,1);
	 CC2i(K) := M(2,2);
	 CC3i(K) := M(1,2);
      end loop;
      if not STrunc then
	 CE1 := MakeCoeff(RW,RW);
	 CE2 := MakeCoeff(RW2,RW2);
	 CE3 := MakeCoeff(RW,RW2);
      end if;
   end SetRW;
   
   procedure Conv(CC: in FPoly1; G: in out Fourier1) is
   begin
      for K in Freq loop
	 G.C(K) := CC(K)*G.C(K);
      end loop;
   end Conv;

   procedure Conv(G1,G2: in out Fourier1) is
      H1,H2: Fourier1;
   begin
      SetZero(G1.R,H1);
      SetZero(G2.R,H2);
      for K in Freq loop
	 H1.C(K) := CC1(K)*G1.C(K)+CC3(K)*G2.C(K);
	 H2.C(K) := CC3(K)*G1.C(K)+CC2(K)*G2.C(K);
      end loop;
      if not STrunc then
	 for K in EFreq loop
	    H1.E(K) := CE1(K)*G1.E(K)+CE3(K)*G2.E(K);
	    H2.E(K) := CE3(K)*G1.E(K)+CE2(K)*G2.E(K);
	 end loop;
      end if;
      Copy(H1,G1);
      Copy(H2,G2);
   end Conv;
   
   procedure ConvInv(G1,G2: in out Fourier1) is
      H1,H2: Fourier1;
   begin
      SetZero(G1.R,H1);
      SetZero(G2.R,H2);
      for K in Freq loop
	 H1.C(K) := CC1i(K)*G1.C(K)+CC3i(K)*G2.C(K);
	 H2.C(K) := CC3i(K)*G1.C(K)+CC2i(K)*G2.C(K);
      end loop;
      Copy(H1,G1);
      Copy(H2,G2);
   end ConvInv;
   
   function ScalProd(U,V: Vector) return Scalar is
      Dim: constant Positive := U'Last(1)/2;
      IM: Matrix(U'Range,U'Range);
   begin
      SetZero(IM);
      for K in 1 .. Dim loop
   	 IM(K,K) := CC1i(K-1);
   	 IM(K+Dim,K) := CC3i(K-1);
   	 IM(K,K+Dim) := CC3i(K-1);
   	 IM(K+Dim,K+Dim) := CC2i(K-1);
      end loop;
      return U*(IM*V);
   end ScalProd;

   function PsiAdjoint(A: Matrix) return Matrix is
      Dim: constant Positive := A'Last(1)/2;
      M,IM: Matrix(A'Range(1),A'Range(2));
   begin
      SetZero(M);
      SetZero(IM);
      for K in 1 .. Dim loop
   	 M(K,K) := CC1(K-1);
   	 M(K+Dim,K) := CC3(K-1);
   	 M(K,K+Dim) := CC3(K-1);
   	 M(K+Dim,K+Dim) := CC2(K-1);
   	 IM(K,K) := CC1i(K-1);
   	 IM(K+Dim,K) := CC3i(K-1);
   	 IM(K,K+Dim) := CC3i(K-1);
   	 IM(K+Dim,K+Dim) := CC2i(K-1);
      end loop;
      return M*Transpose(A)*IM;
   end PsiAdjoint;
   
   function PsiTrans(A: Matrix) return Matrix is
      Dim: constant Positive := A'Last(1)/2;
      IM: Matrix(A'Range(1),A'Range(2));
   begin
      SetZero(IM);
      for K in 1 .. Dim loop
   	 IM(K,K) := CC1i(K-1);
   	 IM(K+Dim,K) := CC3i(K-1);
   	 IM(K,K+Dim) := CC3i(K-1);
   	 IM(K+Dim,K+Dim) := CC2i(K-1);
      end loop;
      return Transpose(A)*IM;
   end PsiTrans;
   
   function COrtho(A: Matrix) return Scalar is
      B: constant Matrix(A'Range(1),A'Range(2)) := OrthoErr(A);
   begin
      return Norm1(PsiTrans(A)*B+PsiTrans(B)*A);
   end COrtho;
   
   function OrthoErr(A: in Matrix) return Matrix is
      B: Matrix(A'Range(1),A'Range(2));
   begin
      B := PsiTrans(A)*A;
      AddConst(-SOne,B);
      return B;
   end OrthoErr;

   procedure MapA(P: in Param; F: in Fourier1; G: in out Fourier1) is
   begin
      G := F*(F*F);
      Conv(CC1,G);
      if P(1) = Scal(1) then G := G/MaxNorm(G); end if;
   end MapA;

   procedure DMapA(P: in Param; F: in Fourier1; H: in out Fourier1) is
      pragma Unreferenced (P);
   begin
      H := SThree*((F*F)*H);
      Conv(CC1,H);
   end DMapA;
   
   procedure MapA(P: in Param; F1,F2: in Fourier1; G1,G2: in out Fourier1) is
      MN: Radius;
   begin
      G1 := F1*(F1*F1);
      G2 := Alpha*F2*(F2*F2);
      Conv(G1,G2);
      if P(1) = Scal(1) then
	 MN := MaxNorm(G1)+MaxNorm(G2);
	 G1 := G1/MN;
      	 G2 := G2/MN;
      end if;
   end MapA;

   procedure DMapA(P: in Param; F1,F2: in Fourier1; H1,H2: in out Fourier1) is
      pragma Unreferenced (P);
   begin
      H1 := SThree*(F1*(F1*H1));
      H2 := (SThree*Alpha)*(F2*(F2*H2));
      Conv(H1,H2);
   end DMapA;
   
   ------------------------------
   
   procedure Pack(A: in Fourier1; V: in out Vector) is
      C: FPoly1 renames A.C;
   begin
      for K in 1 .. V'Last loop
	 Copy(C(K-1),V(K));
      end loop;
   end Pack;

   procedure UnPack(V: in Vector; R: in Radius; A: in out Fourier1) is
      C: FPoly1 renames A.C;
   begin
      SetZero(R,A);
      for K in 1 .. V'Last loop
	 Copy(V(K),C(K-1));
      end loop;
   end UnPack;

   procedure Pack(A1,A2: in Fourier1; V: in out Vector) is
      Dim: constant Positive := V'Last/2;
      C1: FPoly1 renames A1.C;
      C2: FPoly1 renames A2.C;
   begin
      for K in 1 .. Dim loop
	 Copy(C1(K-1),V(K));
	 Copy(C2(K-1),V(K+Dim));
      end loop;
      V(1) := SqrtTwo*V(1);
      V(1+Dim) := SqrtTwo*V(1+Dim);
   end Pack;

   procedure UnPack(V: in Vector; R: in Radius; A1,A2: in out Fourier1) is
      Dim: constant Positive := V'Last/2;
      C1: FPoly1 renames A1.C;
      C2: FPoly1 renames A2.C;
   begin
      SetZero(R,A1);
      SetZero(R,A2);
      for K in 1 .. Dim loop
	 Copy(V(K),C1(K-1));
	 Copy(V(K+Dim),C2(K-1));
      end loop;
      C1(0) := InvSqrtTwo*C1(0);
      C2(0) := InvSqrtTwo*C2(0);
   end UnPack;

   procedure Prod(A: in Matrix; F: in Fourier1; G: in out Fourier1) is
      V: Vector(A'Range(2));
   begin
      Pack(F,V);
      Mult(A,V);
      UnPack(V,F.R,G);
   end Prod;

   procedure Prod(A: in Matrix; F1,F2: in Fourier1; G1,G2: in out Fourier1) is
      V: Vector(A'Range(2));
   begin
      Pack(F1,F2,V);
      Mult(A,V);
      UnPack(V,F1.R,G1,G2);
   end Prod;

   function OpNorm(R: Radius; M: Matrix) return Radius is
      N: Radius := Zero;
      N0,N1: Radius;
      V: Vector(M'Range(2));
      F1,F2: Fourier1;
      function Norm(V: Vector) return Scalar is
      begin
         UnPack(V,R,F1,F2);
	 return Norm1(F1)+Norm1(F2);
      end Norm;
   begin
      for K in V'Range loop
         SetZero(V);
         Assign(1,V(K));
         N0 := Inf(Norm(V));
         Mult(M,V);
         N1 := Sup(Norm(V));
         N := RMax(N1/N0,N);
      end loop;
      return N;
   end OpNorm;
   
   function OpNorm(P: Param; M: Matrix; F1,F2: Fourier1) return Radius is
      N,N0: Radius := Zero;
      V: Vector(M'Range(2));
      G1,G2,H1,H2: Fourier1;
      MaxDeg: constant Positive := M'Last(1)/2;
   begin
      for K in V'Range loop
         SetZero(V);
         Assign(1,V(K));
         UnPack(V,F1.R,G1,G2);
         N0 := Inf(Norm1(G1)+Norm1(G2));
         Mult(M,V);
         UnPack(V,F1.R,H1,H2);
	 DMapA(P,F1,F2,G1,G2);
	 Sub(H1,G1);
	 Sub(H2,G2);
         N  := RMax((MaxNorm(G1)+MaxNorm(G2))/N0,N);
      end loop;
      SetZero(H1);
      SetZero(H2);
      H1.E(MaxDeg+1) := UpInvNormFac(F1.R,MaxDeg+1);
      H2.E(MaxDeg+1) := UpInvNormFac(F2.R,MaxDeg+1);
      DMapA(P,F1,F2,H1,H2);
      N := RMax(MaxNorm(H1)+MaxNorm(H2),N);
      return N;
   end OpNorm;
   
   procedure Contract(P: in Param; F1,F2: in Fourier1; A: in Matrix; H1,H2: in out Fourier1; Err: out Radius) is
      AH1,AH2,F11,F12,F21,F22: Fourier1;
   begin
      Prod(A,H1,H2,AH1,AH2);
      Sum(H1,AH1,F11);
      Sum(H2,AH2,F12);
      Add(F1,F11);
      Add(F2,F12);
      MapA(P,F11,F12,F21,F22);
      Sub(F21,F11);
      Sub(F22,F12);
      Err := MaxNorm(F1)+MaxNorm(F2);
      Sub(F1,F21);
      Sub(F2,F22);
      Diff(F21,AH1,H1);
      Diff(F22,AH2,H2);
   end Contract;

   function DContractNorm(P: Param; F1,F2: Fourier1; A: Matrix) return Radius is
      MaxDeg: constant Positive := 40;
      R,N0,N1: Radius;
      AH1,AH2: Fourier1;
      procedure DContract(H1,H2: in out Fourier1) is
      begin
	 Prod(A,H1,H2,AH1,AH2);
	 Add(AH1,H1);
	 Add(AH2,H2);
	 DMapA(P,F1,F2,H1,H2);
	 Sub(AH1,H1);
	 Sub(AH2,H2);
      end DContract;
      H1,H2: Fourier1;
   begin
      SetZero(H1);
      SetZero(H2);
      H1.E(MaxDeg+1) := UpInvNormFac(F1.R,MaxDeg+1);
      H2.E(MaxDeg+1) := UpInvNormFac(F2.R,MaxDeg+1);
      DContract(H1,H2);
      R := MaxNorm(H1)+MaxNorm(H2);
      if Verbosity>1 then
	 Show1(" ||high+||",R);
      end if;
      
      for M in 0..MaxDeg loop
	 for K in 0..1 loop
	    SetZero(H1);
	    SetZero(H2);
	    if K=0 then
	       Assign(1,H1.C(M));
	       N0 := Inf(Norm1(H1));
	    else
	       Assign(1,H2.C(M));
	       N0 := Inf(Norm1(H2));
	    end if;
	    DContract(H1,H2);
	    N1 := MaxNorm(H1)+MaxNorm(H2);
	    if Verbosity>1 then
	       Show1("K= ",M,False);
	       Show1(" ||DN||= ",N1/N0);
	    end if;
	    R := RMax(R,N1/N0);
	 end loop;
      end loop;
      if Verbosity>1 then
	 Show1("R",R);
      end if;
      return R;
   end DContractNorm;
   
   procedure CheckFix(P: in Param; A: in Matrix; F1,F2: in out Fourier1; KMax: in Radius) is
      --- assuming Flt operations are rounded up
      Rad,Del,N,K: Radius;
      G1,G2: Fourier1;
   begin
      MapA(P,F1,F2,G1,G2);
      Sub(F1,G1);
      Sub(F2,G2);
      Del := MaxNorm(G1)+MaxNorm(G2);
      if Verbosity>1 then
	 Show1("diff",Del);
      end if;
      Rad := (-Del)/(KMax-One);
      N := One+OpNorm(RMax(Rho(F1),Rho(F2)),A);
      Copy(F1,G1);
      Copy(F2,G2);
      AddBall(Rad*N,G1);
      AddBall(Rad*N,G2);
      K := DContractNorm(P,G1,G2,A);
      if K>KMax then
         Show0("Fouriers1.Periodic.CheckFix failed.");
      else
	 Show0("Fouriers1.Periodic.CheckFix OK.");
      end if;
      Rad := (-Del)/(K-One);    -- radius for contraction
      AddBall(Rad*N,F1);
      AddBall(Rad*N,F2);
   end CheckFix;
   
   function Val(P: Fourier1; S1: Quadr) return Quadr is
      S2: Quadr;
      C: FPoly1 renames P.C;
      D: constant Integer := PDeg;
      Cos1: constant Quadr := Cos(S1);
      Sin1: constant Quadr := Sin(S1);
      CC,SS,CTmp,STmp,Tmp: Quadr;
   begin
      SetZero(CC);
      SetZero(SS);
      SetZero(S2);
      if not (STrunc or else Flt_Vectors.IsZero(P.E)) then
	 BallAt0(UpSum(P.E),S2(0));
      end if;
      CC(0) := C(D);
      SS(0) := C(-D);
      for I in reverse 0 .. D-1 loop
        Prod(Cos1,CC,CTmp);
        Prod(Sin1,SS,Tmp);
        Add(Tmp,CTmp);
        Prod(Sin1,CC,Tmp);
        Prod(Cos1,SS,STmp);
        Sub(Tmp,STmp);
        CC := CTmp;
	CC(0) := CC(0)+C(I);
	exit when I=0;
	SS := STmp;
	SS(0) := SS(0)+C(-I);
      end loop;
      Add(CC,S2);
      return S2;
   end Val;
   
   -------------------------------------- numerical
   
   procedure DerMatrix(P: in Param; F: in Fourier1; A: in out Matrix) is
      --- use only with numeric scalars
      U: Vector(A'Range(2));
      V: Vector(A'Range(1));
      H: Fourier1;
   begin
      for K in U'Range loop
         SetZero(U);
         Assign(1,U(K));
         UnPack(U,F.R,H);
         DMapA(P,F,H);
         Pack(H,V);
         SetColumn(K,V,A);
      end loop;
   end DerMatrix;

   procedure DerMatrix(P: in Param; F1,F2: in Fourier1; A: in out Matrix) is
      --- use only with numeric scalars
      U: Vector(A'Range(2));
      V: Vector(A'Range(1));
      H1,H2: Fourier1;
   begin
      for K in U'Range loop
         SetZero(U);
         Assign(1,U(K));
         UnPack(U,F1.R,H1,H2);
         DMapA(P,F1,F2,H1,H2);
         Pack(H1,H2,V);
         SetColumn(K,V,A);
      end loop;
   end DerMatrix;

   procedure ContrMatrix(P: in Param; F: in Fourier1; A: in out Matrix) is
      --- use only with numeric scalars
   begin
      DerMatrix(P,F,A);
      AddConst(-Scal(One),A);
      Invert(A);              -- inv(DG-I)
      AddConst(Scal(One),A);  -- inv(DG-I)+I
      Neg(A);                 -- -I-inv(DG-I)
   end ContrMatrix;
   
   procedure ContrMatrix(P: in Param; F1,F2: in Fourier1; A: in out Matrix) is
      --- use only with numeric scalars
   begin
      DerMatrix(P,F1,F2,A);
      AddConst(-Scal(One),A);
      Invert(A);              -- inv(DG-I)
      AddConst(Scal(One),A);  -- inv(DG-I)+I
      Neg(A);                 -- -I-inv(DG-I)
   end ContrMatrix;
   
   procedure ShowEigen(A: in Matrix) is
      use SSV_RNum;
      Wr,Wi: Vector(A'Range(1));
      Vr,Vi: Matrix(A'Range(1),A'Range(2));
   begin
      --- use only with numeric scalars
      Eigen(A,Wr,Wi,Vr,Vi);
      for I in Vr'First..10 loop
         Show2("  E" & Strng(I,3),Approx(Wr(I)),Approx(Wi(I)));
      end loop;
   end ShowEigen;
   
   procedure Contract0(P: in Param; A: in Matrix; F: in out Fourier1; Err: out Radius) is
      F0: constant Fourier1 := F;
      H,AH: Fourier1;
   begin
      MapA(P,F0,F);
      Diff(F,F0,H);
      Err := MaxNorm(H);
      Prod(A,H,AH);
      Add(AH,F);
   end Contract0;

   procedure Contract0(P: in Param; A: in Matrix; F1,F2: in out Fourier1; Err: out Radius) is
      F01: constant Fourier1 := F1;
      F02: constant Fourier1 := F2;
      H1,H2,AH1,Ah2: Fourier1;
   begin
      MapA(P,F01,F02,F1,F2);
      Diff(F1,F01,H1);
      Diff(F2,F02,H2);
      Err := MaxNorm(H1)+MaxNorm(H2);
      Prod(A,H1,H2,AH1,AH2);
      Add(AH1,F1);
      Add(AH2,F2);
   end Contract0;

   procedure ApproxFix(P: in Param; F: in out Fourier1; A: in out Matrix; Eps: in Radius) is
      Iter: constant Integer := 100;
      MaxBad: constant Integer := 5;
      MaxErr: constant Radius := 1.0E+9;
      Bad: Integer := 0;
      MinErr,LastErr: Radius := MaxErr;
      Err: Radius := Zero;
      G: Fourier1 := F;
   begin
      for I in 1 .. Iter loop
         ContrMatrix(P,G,A);
         Contract0(P,A,G,Err);
         Show1("ApproxFix: " & Strng(I),Err);
         if Err<LastErr then
            Bad := 0;
            if Err<MinErr then
               Copy(G,F);
               if Err<Eps then return; end if;
               MinErr := Err;
            end if;
         else
            Bad := Bad+1;
            exit when (Err>MaxErr) or (Bad>MaxBad);
         end if;
         LastErr := Err;
      end loop;
      Show0("Fouriers1.Green.ApproxFix getting tired");
   end ApproxFix;
   
   procedure ApproxFix(P: in Param; F1,F2: in out Fourier1; A: in out Matrix; Eps: in Radius) is
      Iter: constant Integer := 100;
      MaxBad: constant Integer := 5;
      MaxErr: constant Radius := 1.0E+9;
      Bad: Integer := 0;
      MinErr,LastErr: Radius := MaxErr;
      Err: Radius := Zero;
      G1: Fourier1 := F1;
      G2: Fourier1 := F2;
   begin
      for I in 1 .. Iter loop
         ContrMatrix(P,G1,G2,A);
         Contract0(P,A,G1,G2,Err);
         Show1("ApproxFix: " & Strng(I),Err);
         if Err<LastErr then
            Bad := 0;
            if Err<MinErr then
               Copy(G1,F1);
               Copy(G2,F2);
               if Err<Eps then return; end if;
               MinErr := Err;
            end if;
         else
            Bad := Bad+1;
            exit when (Err>MaxErr) or (Bad>MaxBad);
         end if;
         LastErr := Err;
      end loop;
      Show0("Fouriers1.Green.ApproxFix getting tired");
   end ApproxFix;
   
   procedure Plot(F: in Fourier1; L: Positive := 50) is
      Pi2: constant Flt := 6.283185307179586476925286766559;
      X,Y: Flt;
      File: File_Type;
   begin
      if Verbosity>0 then Ada.Text_IO.Put_Line("Writing out.xy"); end if;
      Create(File,Out_File,"out.xy");
      for M in 1 .. L loop
         X := Pi2*Flt(M)/Flt(L);
         Y := Approx(Val(F,Scal(X)));
         Plot2(File,X,Y);
      end loop;
      Close(File);
   end Plot;
   
   procedure Plot(F1,F2: in Fourier1; L: Positive := 50) is
      Pi2: constant Flt := 6.283185307179586476925286766559;
      Th,Z: Scalar;
      H1,H2: Fourier1;
      File: File_Type;
   begin
      if Verbosity>0 then Ada.Text_IO.Put_Line("Writing out1.xy and out2.xy"); end if;
      Create(File,Out_File,"out1.xy");
      H1 := F1*(F1*F1);
      H2 := F2*(F2*F2);
      Conv(MakeCoeff(RW,RW),H1);
      Conv(MakeCoeff(RW,RW2),H2);
      for N in 0 .. L loop
	 Th := Pi2*Scal(N)/Scal(L);
	 Z := Val(H1+Alpha*H2,Th);
	 Plot2(File,Approx(Th),Approx(Z));
      end loop;
      New_Line(File);
      Close(File);
      Create(File,Out_File,"out2.xy");
      H1 := F1*(F1*F1);
      H2 := F2*(F2*F2);
      Conv(MakeCoeff(RW2,RW),H1);
      Conv(MakeCoeff(RW2,RW2),H2);
      for N in 0 .. L loop
	 Th := Pi2*Scal(N)/Scal(L);
	 Z := Val(H1+Alpha*H2,Th);
	 Plot2(File,Approx(Th),Approx(Z));
      end loop;
      New_Line(File);
      Close(File);
      Create(File,Out_File,"out1mag.xy");
      H1 := F1*(F1*F1);
      H2 := F2*(F2*F2);
      Conv(MakeCoeff(RW,RW),H1);
      Conv(MakeCoeff(RW,RW2),H2);
      for N in 0 .. L loop
	 Th := 0.5*Scal(N-25)/Scal(L);
	 Z := Val(H1+Alpha*H2,Th);
	 Plot2(File,Approx(Th),Approx(Z));
      end loop;
      New_Line(File);
      Close(File);
   end Plot;

   procedure Plot3(F: in Fourier1; L: Positive := 50) is
      Pi2: constant Flt := 6.283185307179586476925286766559;
      R,Th,Z: Scalar;
      File: File_Type;
      H: Fourier1;
   begin
      if Verbosity>0 then Ada.Text_IO.Put_Line("Writing out.xyz"); end if;
      Create(File,Out_File,"out.xyz");
      H := F*F*F;
      for M in 1 .. L loop
	 R := Scal(M)/Scal(L);
	 Conv(MakeCoeff(R,RW),H);
         for N in 0 .. L loop
	    Th := Pi2*Scal(N)/Scal(L);
            Z := Val(H,Th);
            Plot3(File,Approx(R*Cos(Th)),Approx(R*Sin(Th)),Approx(Z));
         end loop;
         New_Line(File);
      end loop;
      Close(File);
   end Plot3;
   
   procedure Plot3(F1,F2: in Fourier1; L: Positive := 50) is
      Pi2: constant Flt := 6.283185307179586476925286766559;
      R,Th,Z: Scalar;
      File: File_Type;
      H1,H2: Fourier1;
   begin
      if Verbosity>0 then Ada.Text_IO.Put_Line("Writing out.xyz "); end if;
      Create(File,Out_File,"out.xyz");
      for M in 1 .. L loop
      	 R := Scal(M)/Scal(L);
	 H1 := F1*(F1*F1);
	 H2 := F2*(F2*F2);
      	 Conv(MakeCoeff(R,RW),H1);
      	 Conv(MakeCoeff(R,RW2),H2);
         for N in 0 .. L loop
      	    Th := Pi2*Scal(N)/Scal(L);
            Z := Val(H1+Alpha*H2,Th);
            Plot3(File,Approx(R*Cos(Th)),Approx(R*Sin(Th)),Approx(Z));
         end loop;
         New_Line(File);
      end loop;
      Close(File);
   end Plot3;
   
   procedure Write(FileName: in String; P1,P2: in Fourier1; Decimal: in Boolean := False) is
      F: File_Type;
   begin
      if Verbosity>0 then Show0("Writing " & FileName); end if;
      Create(F,Out_File,FileName);
      Put(F,P1,Decimal);
      Put(F,P2,Decimal);
      Close(F);
   end Write;
   
   procedure Read(FileName: in String; P1,P2: in out Fourier1; Decimal: in Boolean := False) is
      F: File_Type;
   begin
      if Verbosity>0 then Show0("Reading " & FileName); end if;
      Open(F,In_File,FileName);
      Get(F,P1,Decimal);
      Get(F,P2,Decimal);
      Close(F);
   end Read;

   function MakeCoeff(R,Rho: Scalar) return FPoly1 is
      L: constant Scalar := R*Rho;
      M: constant Scalar := Min(R/Rho,Rho/R);
      C: FPoly1;
   begin
      SetZero(C);
      C(0) := -Log(Max(R,Rho));
      for K in 1..PDeg loop
   	 C(K) := Inv(Scal(2*K))*(M**K-L**K);
      end loop;
      return C;
   end MakeCoeff;
   
   function MakeCoeff(R,Rho: Scalar) return Errs1 is
      L: constant Scalar := R*Rho;
      M: constant Scalar := Min(R/Rho,Rho/R);
      C: Errs1;
   begin
      C(0) := MaxNorm(Log(Max(R,Rho)));
      for K in 1..EDeg loop
   	 C(K) := MaxNorm(Inv(Scal(2*K))*(L**K-M**K));
	 C(-K) := Zero;		 
      end loop;
      return C;
   end MakeCoeff;

end Fouriers1.Green;
