with ScalVectors, ScalVectors.RNum, Quadrs;

pragma Elaborate_All (ScalVectors.RNum,Quadrs);

generic

   with package SSV is new Scalvectors (Scalar => Scalar, SM => SM, SV => SV);

package Fouriers1.Green is

   package SSV_RNum is new SSV.RNum;
   package Quad is new Quadrs (Scalar => Scalar, SM => SM, SV => SV);
   use SM,SV,SSV,Quad;
   
   subtype Param is Vector2;
   
--- existence
   procedure SetRW(R: in Flt);
   procedure SetRW(R1,R2,A: in Flt);
   procedure MapA(P: in Param; F: in Fourier1; G: in out Fourier1);
   procedure DMapA(P: in Param; F: in Fourier1; H: in out Fourier1);
   procedure MapA(P: in Param; F1,F2: in Fourier1; G1,G2: in out Fourier1);
   procedure DMapA(P: in Param; F1,F2: in Fourier1; H1,H2: in out Fourier1);
   
   procedure Pack(A: in Fourier1; V: in out Vector);
   procedure UnPack(V: in Vector; R: in Radius; A: in out Fourier1);
   procedure Pack(A1,A2: in Fourier1; V: in out Vector);
   procedure UnPack(V: in Vector; R: in Radius; A1,A2: in out Fourier1);
   procedure Prod(A: in Matrix; F: in Fourier1; G: in out Fourier1);
   procedure Prod(A: in Matrix; F1,F2: in Fourier1; G1,G2: in out Fourier1);
   function OpNorm(R: Radius; M: Matrix) return Radius;
   function OpNorm(P: Param; M: Matrix; F1,F2: Fourier1) return Radius;
   procedure Contract(P: in Param; F1,F2: in Fourier1; A: in Matrix; H1,H2: in out Fourier1; Err: out Radius);
   function DContractNorm(P: Param; F1,F2: Fourier1; A: Matrix) return Radius;
   procedure CheckFix(P: in Param; A: in Matrix; F1,F2: in out Fourier1; KMax: in Radius);
   function Val(P: Fourier1; S1: Quadr) return Quadr;
   function PsiAdjoint(A: Matrix) return Matrix;
   function PsiTrans(A: Matrix) return Matrix;
   function OrthoErr(A: in Matrix) return Matrix;
   function COrtho(A: Matrix) return Scalar;
   function ScalProd(U,V: Vector) return Scalar;
   procedure Conv(CC: in FPoly1; G: in out Fourier1);
   procedure Conv(G1,G2: in out Fourier1);
   procedure ConvInv(G1,G2: in out Fourier1);
   
--- numerical
   procedure DerMatrix(P: in Param; F: in Fourier1; A: in out Matrix);
   procedure DerMatrix(P: in Param; F1,F2: in Fourier1; A: in out Matrix);
   procedure ContrMatrix(P: in Param; F: in Fourier1; A: in out Matrix);
   procedure ContrMatrix(P: in Param; F1,F2: in Fourier1; A: in out Matrix);
   procedure ShowEigen(A: in Matrix);
   procedure Contract0(P: in Param; A: in Matrix; F: in out Fourier1; Err: out Radius);
   procedure Contract0(P: in Param; A: in Matrix; F1,F2: in out Fourier1; Err: out Radius);
   procedure ApproxFix(P: in Param; F: in out Fourier1; A: in out Matrix; Eps: in Radius);
   procedure ApproxFix(P: in Param; F1,F2: in out Fourier1; A: in out Matrix; Eps: in Radius);
   procedure Plot(F: in Fourier1; L: Positive := 50);
   procedure Plot(F1,F2: in Fourier1; L: Positive := 50);
   procedure Plot3(F: in Fourier1; L: Positive := 50);
   procedure Plot3(F1,F2: in Fourier1; L: Positive := 50);
   procedure Write(FileName: in String; P1,P2: in Fourier1; Decimal: in Boolean := False);
   procedure Read(FileName: in String; P1,P2: in out Fourier1; Decimal: in Boolean := False);

private

   function MakeCoeff(R,Rho: Scalar) return FPoly1;
   function MakeCoeff(R,Rho: Scalar) return Errs1;

   SqrtTwo: Scalar := Sqrt(Scal(2));
   InvSqrtTwo: Scalar := Sqrt(Scal(Half));
   RW,RW2,Alpha: Scalar;           -- radius(es) of the support and coefficient of the second one
   CC1,CC2,CC3,CC1i,CC2i,CC3i:  FPoly1;
   CE1,CE2,CE3:  Errs1;
   
end Fouriers1.Green;
