with Ada.Text_IO, Logicals, Globals, Rationals, Flts;
use Ada.Text_IO, Logicals, Globals, Rationals, Flts;
with Reps, Matrices, Vectors, ScalVectors, ScalVectors.RNum;
with Fouriers1, Fouriers1.Green, NumFouriers1;

pragma Elaborate_All (Ada.Text_IO,Logicals,Globals,Rationals,Flts);
pragma Elaborate_All (Reps,Matrices,ScalVectors.RNum,Fouriers1.Green,NumFouriers1);

generic

  PDeg: in Positive;

-------------------- begin_include scalars Scalar

  type Scalar is private;

--- basic
  with function Info(Dummy: Scalar) return Scalar_Info is <>;
  with function IsSharp(S: Scalar) return Boolean is <>;
  with function IsNumbers(S: Scalar) return Boolean is <>;
  with function IsZero(S: Scalar) return Boolean is <>;
  with function "="(S1,S2: Scalar) return Boolean is <>;
  with procedure SetZero(S: in out Scalar) is <>;
  with procedure Copy(S1: in Scalar; S2: in out Scalar) is <>;
  with procedure Swap(S1,S2: in out Scalar) is <>;

--- sets
  with function Center0(S: Scalar) return Boolean is <>;
  with function Contains0(S: Scalar) return Logical is <>;
  with function Contains(S1,S2: Scalar) return Logical is <>; -- S1 constains S2
  with procedure BallAt0(R: in Flt; S: in out Scalar) is <>;
  with function BallAt0(R: Flt) return Scalar is <>;
  with procedure DiskAt0(R: in Flt; S: in out Scalar) is <>;
  with function DiskAt0(R: Flt) return Scalar is <>;
  with procedure ToErr(S: in out Scalar) is <>;
  with procedure ToErr(S1: in Scalar; S2: in out Scalar) is <>;
  with function ToErr(S: Scalar) return Scalar is <>;
  with procedure Center(S: in out Scalar) is <>;
  with procedure Center(S1: in Scalar; S2: in out Scalar) is <>;
  with function Center(S: Scalar) return Scalar is <>;
  with procedure CenterDisk(S: in out Scalar) is <>;
  with procedure CenterDisk(S1: in Scalar; S2: in out Scalar) is <>;
  with function CenterDisk(S: Scalar) return Scalar is <>;
  with procedure ModCenter(S: in out Scalar) is <>;
  with procedure ModCenter(S1: in Scalar; S2: in out Scalar) is <>;
  with function ModCenter(S: Scalar) return Scalar is <>;
  with procedure ModNumbers(S: in out Scalar) is <>;
  with procedure ModNumbers(S1: in Scalar; S2: in out Scalar) is <>;
  with function ModNumbers(S: Scalar) return Scalar is <>;
  with procedure ErrMult(R: in Radius; S: in out Scalar) is <>;
  with procedure Union(S1: in Scalar; S2: in out Scalar) is <>;
  with function Union(S1,S2: Scalar) return Scalar is <>;
  with procedure Intersection(S1: in Scalar; S2: in out Scalar; Empty: out Logical) is <>;

--- order
  with function Sign(S: Scalar) return Integer is <>;
  with function Compare(S1,S2: Scalar) return Integer is <>;
  with function "<"(S1,S2: Scalar) return Boolean is <>;
  with function "<="(S1,S2: Scalar) return Boolean is <>;
  with function ">"(S1,S2: Scalar) return Boolean is <>;
  with function ">="(S1,S2: Scalar) return Boolean is <>;
  with procedure Min(S1: in Scalar; S2: in out Scalar) is <>;
  with procedure Min(S1,S2: in Scalar; S3: in out Scalar) is <>;
  with function Min(S1,S2: Scalar) return Scalar is <>;
  with procedure Max(S1: in Scalar; S2: in out Scalar) is <>;
  with procedure Max(S1,S2: in Scalar; S3: in out Scalar) is <>;
  with function Max(S1,S2: Scalar) return Scalar is <>;
  with function Inf(S: Scalar) return Flt is <>; -- inf of real part
  with function Sup(S: Scalar) return Flt is <>; -- sup of real part

--- arithmetic
  with procedure Neg(S: in out Scalar) is <>;
  with procedure Neg(S1: in Scalar; S2: in out Scalar) is <>;
  with function "-"(S: Scalar) return Scalar is <>;
  with procedure Add(I: in Integer; S: in out Scalar) is <>;
  with procedure Add(S1: in Scalar; S2: in out Scalar) is <>;
  with procedure Sum(S1,S2: in Scalar; S3: in out Scalar) is <>;
  with function "+"(S1,S2: Scalar) return Scalar is <>;
  with procedure Sub(S1: in Scalar; S2: in out Scalar) is <>;
  with procedure Diff(S1,S2: in Scalar; S3: in out Scalar) is <>;
  with function "-"(S1,S2: Scalar) return Scalar is <>;
  with procedure Mult(R: in Flt; S,Tmp: in out Scalar) is <>;
  with procedure Mult(R: in Flt; S: in out Scalar) is <>;
  with procedure Prod(R: in Flt; S1: in Scalar; S2: in out Scalar) is <>;
  with function "*"(R: Flt; S: Scalar) return Scalar is <>;
  with procedure AddProd(R: in Flt; S1: in Scalar; S2,Tmp: in out Scalar) is <>;
  with procedure AddProd(R: in Flt; S1: in Scalar; S2: in out Scalar) is <>;
  with procedure Mult(Q: in Rational; S: in out Scalar) is <>;
  with procedure Mult(S1: in Scalar; S2: in out Scalar) is <>;
  with procedure Prod(S1,S2: in Scalar; S3: in out Scalar) is <>;
  with function "*"(S1,S2: Scalar) return Scalar is <>;
  with procedure AddProd(S1,S2: in Scalar; S3,Tmp: in out Scalar) is <>;
  with procedure AddProd(S1,S2: in Scalar; S3: in out Scalar) is <>;
  with procedure SubProd(S1,S2: in Scalar; S3,Tmp: in out Scalar) is <>;
  with procedure Div(R: in Flt; S,Tmp: in out Scalar) is <>;
  with procedure Div(R: in Flt; S: in out Scalar) is <>;
  with procedure Quot(S1: in Scalar; R: in Flt; S2: in out Scalar) is <>;
  with function "/"(S: Scalar; R: Flt) return Scalar is <>;
  with procedure Div(S1: in Scalar; S2: in out Scalar) is <>;
  with procedure Quot(S1,S2: in Scalar; S3: in out Scalar) is <>;
  with function "/"(S1,S2: Scalar) return Scalar is <>;
  with procedure Inv(S1: in Scalar; S2: in out Scalar) is <>;
  with function Inv(S: Scalar) return Scalar is <>;
  with function "**"(S: Scalar; I: Integer) return Scalar is <>;

--- fun
  with function IsReal(S: Scalar) return Boolean is <>;
  with procedure Adjoint(S: in out Scalar) is <>;
  with procedure Adjoint(S1: in Scalar; S2: in out Scalar) is <>;
  with function Adjoint(S: Scalar) return Scalar is <>;
  with procedure Real_Part(S: in out Scalar) is <>;
  with procedure Imag_Part(S: in out Scalar) is <>;
  with procedure Eval0(S: in out Scalar) is <>;
  with function "abs"(S: Scalar) return Scalar is <>;
  with procedure Norm(S1: in Scalar; S2: in out Scalar) is <>;
  with function Norm(S: Scalar) return Scalar is <>;
  with function MaxNorm(S: Scalar) return Radius is <>;
  with procedure Sqr(S: in out Scalar) is <>;
  with function Sqr(S: Scalar) return Scalar is <>;
  with procedure Sqrt(S1: in Scalar; S2: in out Scalar) is <>;
  with function Sqrt(S: Scalar) return Scalar is <>;
  with procedure Root(K: in Positive; S1: in Scalar; S2: in out Scalar) is <>;
  with function Root(K: Positive; S: Scalar) return Scalar is <>;
  with procedure Exp(S1: in Scalar; S2: in out Scalar) is <>;
  with function Exp(S: Scalar) return Scalar is <>;
  with procedure ArcCos(S1: in Scalar; S2: in out Scalar) is <>;
  with function ArcCos(S: Scalar) return Scalar is <>;
  with procedure ArcSin(S1: in Scalar; S2: in out Scalar) is <>;
  with function ArcSin(S: Scalar) return Scalar is <>;
  with procedure Cos(S1: in Scalar; S2: in out Scalar) is <>;
  with function Cos(S: Scalar) return Scalar is <>;
  with procedure Sin(S1: in Scalar; S2: in out Scalar) is <>;
  with function Sin(S: Scalar) return Scalar is <>;
  with function Pi return Scalar is <>;
  with procedure Log(S1: in Scalar; S2: in out Scalar) is <>;
  with function Log(S: Scalar) return Scalar is <>;
  with procedure Simple_Random(S: in out Scalar) is <>;
  with procedure QPower(Q: in Rational; S1: in Scalar; S2: in out Scalar) is <>;
  with procedure Roots2(B,C: in Scalar; U1,U2,V: in out Scalar) is <>;
  with procedure Roots3(B,C,D: in Scalar; U0,U1,U2,V: in out Scalar) is <>;

--- conversion and i/o
  with function Approx(S: Scalar) return Flt is <>;
  with procedure Assign(K: in Integer; S: in out Scalar) is <>;
  with procedure Assign(Q: in Rational; S: in out Scalar) is <>;
  with procedure Assign(R: in Flt; S: in out Scalar) is <>;
  with function Scal(I: Integer) return Scalar is <>;
  with function Scal(Q: Rational) return Scalar is <>;
  with function Scal(R: Flt) return Scalar is <>;
  with procedure Enclose(R1,R2: in Flt; S: in out Scalar) is <>;
  with function Enclose(R1,R2: Flt) return Scalar is <>;
  with procedure Show1(N: in String; S: in Scalar; Whatever: in Boolean := True) is <>;
  with procedure Show2(N: in String; S1,S2: in Scalar; Whatever: in Boolean := True) is <>;
  with procedure Put(F: in File_Type; S: in Scalar; Decimal: in Boolean := False) is <>;
  with procedure Get(F: in File_Type; S: in out Scalar; Decimal: in Boolean := False) is <>;

--- misc
  with function Get_Precision(R: Scalar) return Positive is <>;
  with procedure Proper_Rounding(Dummy: in Scalar) is <>;
  with procedure Free_Cache(S: in Scalar) is <>;
-------------------- end_include scalars
  
  with procedure PutNum(S: in Scalar; R: in out Reps.Rep) is <>;
  with procedure GetNum(R: in Reps.Rep; S: in out Scalar) is <>;

package Greens is
  
  package SM is new Matrices (Scalar => Scalar);
  package SV is new Vectors (Scalar => Scalar, Component => Scalar, SM => SM);
  package SSV is new ScalVectors (Scalar => Scalar, SM => SM, SV => SV);
  package SSV_RNum is new SSV.RNum;
  package SFouriers1 is new Fouriers1 (PDeg => PDeg, Scalar => Scalar, SM => SM, SV => SV);
  package SFouriers1_SV_More renames SFouriers1.SV_More;
  package SFouriers1_Green is new SFouriers1.Green (SSV => SSV);
  package SFouriers1_Green_Quad renames SFouriers1_Green.Quad;
  package NSSV_RNum renames SFouriers1_Green.SSV_RNum;
  
  subtype SMatrix is SM.Matrix;
  subtype SVector is SV.Vector;
  subtype SDiag is SSV.Diag;
  subtype SParam is SFouriers1_Green.Param;
  subtype SFPoly1 is SFouriers1.FPoly1;
  subtype SErrs1 is SFouriers1.Errs1;
  subtype SFourier1 is SFouriers1.Fourier1;
  
  procedure Set_Scalar_Precision(Binary_Digs: in Natural := 0);
  procedure Set_Numeric_Precision(Binary_Digs: in Natural := 0);
  procedure Set_Precisions(Binary_Digs: in Positive);
  procedure Set_Fourier_Parallel(P: in Boolean := True);
  
  procedure Show_Param;
  procedure SetParam(R1,R2,A: in Flt);
  procedure CheckFix(Dim: in Positive; F1,F2: in out SFourier1; KMax: in Radius);
  procedure CheckSymm(F: in SFourier1; Div: in Positive);
  procedure CheckEigen(Dim: in Positive; F1,F2: in SFourier1);

--- purely numerical  
  procedure Find_Fix(Rho: in Flt; Cas: in Natural);
  procedure Find_Fix(Cas: in Natural);
  
private
  
  package Greens_NumFouriers1 is new NumFouriers1 (PDeg => PDeg);
  use Greens_NumFouriers1;
  
  function Effective_Precision(Binary_Digs: Positive) return Positive;
  procedure Relax_Checks;
  procedure Restore_Checks;

  procedure S2N(S: in Scalar; N: in out Numeric) renames PutNum;
  procedure N2S(N: in Numeric; S: in out Scalar) renames GetNum;
  procedure S2N(S: in SMatrix; N: in out NMatrix);
  procedure N2S(N: in NMatrix; S: in out SMatrix);
  procedure S2N(S: in SVector; N: in out NVector);
  procedure N2S(N: in NVector; S: in out SVector);
  procedure N2S(N: in NVector; S: in out SDiag);
  
  procedure S2N(S: in SFourier1; N: in out NFourier1);
  procedure N2S(N: in NFourier1; S: in out SFourier1);
  
  SZero: constant Scalar := Scal(0);
  SHalf: constant Scalar := Scal(Half);
  SOne:  constant Scalar := Scal(1);
  Trunc:  constant Boolean  := Info(SZero).IsNumeric;
  
  Scalar_Precision: Positive  := Initial_Precision;
  Numeric_Precision: Positive := Initial_Precision;
  Fourier_Parallel:  Boolean  := False;
  
------------------------------------------------------------------
--- the real stuff -----------------------------------------------
------------------------------------------------------------------
  
  procedure Num_ContrMatrix(P: in SParam; F1,F2: in SFourier1; A: in out SMatrix);
  procedure Num_DerMatrix(P: in SParam; F1,F2: in SFourier1; A: in out SMatrix);
  procedure Num_EigenVec(A: in SMatrix; Vr,Vi: in out SMatrix);
  
end Greens;
