
package body ImproveRoots is
  
  use ScalNewton;
  
  procedure ImproveRoot2(B,C: in Scalar; U: in out Scalar; Steps: in Natural := 64; Parallel: in Boolean := False) is
    procedure F(X: in Scalar; Y: in out Scalar) is
    begin
      Sum(B,X,Y);
      Mult(X,Y);
      Add(C,Y);  --- Y := C+X*(B+X));
    end F;
    procedure DF(X: in Scalar; Y: in out Scalar) is
    begin
      Sum(B,X,Y);
      Add(X,Y);  --- Y := B+2*X
    end DF;
    procedure Improve is new ImproveZero(F => F, DF => DF);
  begin
    Improve(U,Steps,Parallel);
  end ImproveRoot2;
  
  procedure ImproveRoots2(B,C: in Scalar; U1,U2: in out Scalar; Steps: in Natural := 64; Parallel: in Boolean := False) is
  begin
    if Parallel then
      declare
        task Improve_U1 is end Improve_U1;
        task body Improve_U1 is begin ImproveRoot2(B,C,U1,Steps,True); end;
      begin
        ImproveRoot2(B,C,U2,Steps,True);
      end;
    else
      ImproveRoot2(B,C,U1,Steps,False);
      ImproveRoot2(B,C,U2,Steps,False);
    end if;
  end ImproveRoots2;
  
  procedure ImproveRoot3(B,C,D: in Scalar; U: in out Scalar; Steps: in Natural := 64; Parallel: in Boolean := False) is
    procedure F(X: in Scalar; Y: in out Scalar) is
    begin
      Sum(B,X,Y);
      Mult(X,Y);
      Add(C,Y);
      Mult(X,Y);
      Add(D,Y);  --- Y := D+X*(C+X*(B+X));
    end F;
    procedure DF(X: in Scalar; Y: in out Scalar) is
    begin
      Sum(X,X,Y);
      Add(X,Y);
      Add(B,Y);
      Add(B,Y);
      Mult(X,Y);
      Add(C,Y);  --- Y := C+X*(2*B+3*X);
    end DF;
    procedure Improve is new ImproveZero(F => F, DF => DF);
  begin
    Improve(U,Steps,Parallel);
  end ImproveRoot3;
  
  procedure ImproveRoots3(B,C,D: in Scalar; U1,U2: in out Scalar; Steps: in Natural := 64; Parallel: in Boolean := False) is
  begin
    if Parallel then
      declare
        task Improve_U1 is end Improve_U1;
        task body Improve_U1 is begin ImproveRoot3(B,C,D,U1,Steps,True); end;
      begin
        ImproveRoot3(B,C,D,U2,Steps,True);
      end;
    else
      ImproveRoot3(B,C,D,U1,Steps,False);
      ImproveRoot3(B,C,D,U2,Steps,False);
    end if;
  end ImproveRoots3;
  
end ImproveRoots;
